--
--              By downloading this software ("the Program") you agree to the
--              following  LIMITED WARRANTY AND DISCLAMER. 
--              The Program is provided to you "as is". 
--              Silicon and Software Systems Limited (S3) make, and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and S3 specifically disclaims any
--              implied warranties of merchantability, non-infringement, or
--              fitness for a particular purpose. S3 does not warrant that
--              the functions contained in the Program will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Program 
--              will be corrected. Furthermore, S3 does not warrant or
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will S3 be liable for
--              any loss of data, lost profits, cost or procurement of 
--              substitute goods or services, or for any special, incidental,
--              consequential, or indirect damages arising from the use or 
--              operation of the Program however caused and on any theory of
--              liability. This limitation will apply even if S3 has been 
--              advised of the possibility of such damage. This limitation 
--              shall apply not-withstanding the failure of the essential 
--              purpose of any limited remedies herein.
--
--              Copyright  2005 SILICON AND SOFTWARE SYSTEMS LIMITED
--              All rights reserved-- Should get a legal disclaimer
--
--------------------------------------------------------------------------------
--
-- Silicon & Software Systems Development Record (CVS)
--
--------------------------------------------------------------------------------
--
--        $RCSfile: gnat.package.vhdl,v $  
--       $Revision: 1.1 $  
--           $Date: 2005/03/09 17:24:37 $  
--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--
-- Project        : Gnat
-- Project Code   : INT2004_57b
--
-- Block Name     : Gnat 
--
-- Description    : Package for customising Gnat component User Register
--
--    
--    User Register
--    -------------------------------------------------------------------------------------------------------
--    |                      |                           |                                                  |
--    |  Peripheral ID       | Opcode (Rd/Wr etc)        |              Data                                |
--    |                      |                           |                                                  |
--    -------------------------------------------------------------------------------------------------------
--    <-- c_PeriphID_width --><- c_PeriphOpCode_width -><----------- c_periph_data_width -----------------> 
--    <-------------- c_UserRegSize ------------------------------------------------------------------------>
--
--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;


package gnat_pkg is

-- Edit these constants to re-define the User Register Width and the widths of the fields
-- These should match the definition in GNAT-Setup.tcl
   constant c_PeriphID_width             : integer := 4 ;
   constant c_PeriphOpCode_width         : integer := 1 ;
   constant c_periph_data_width          : integer := 8;

-- Editing the following section is not recommended 
   constant c_num_peripherals            : integer := (2**c_PeriphID_width) ;
   constant c_UserRegSize                : integer := (c_PeriphID_width + c_PeriphOpCode_width + c_periph_data_width);
  
   type t_gnat_RdData is array ((c_num_peripherals - 1) downto 0) of std_logic_vector((c_periph_data_width - 1) downto 0);

end package gnat_pkg;


--------------------------------------------------------------------------------
-- Package Body Description                                                   --
--------------------------------------------------------------------------------
package body gnat_pkg is

end package body gnat_pkg;

