--
--              By downloading this software ("the Program") you agree to the
--              following  LIMITED WARRANTY AND DISCLAMER. 
--              The Program is provided to you "as is". 
--              Silicon and Software Systems Limited (S3) make, and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and S3 specifically disclaims any
--              implied warranties of merchantability, non-infringement, or
--              fitness for a particular purpose. S3 does not warrant that
--              the functions contained in the Program will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Program 
--              will be corrected. Furthermore, S3 does not warrant or
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will S3 be liable for
--              any loss of data, lost profits, cost or procurement of 
--              substitute goods or services, or for any special, incidental,
--              consequential, or indirect damages arising from the use or 
--              operation of the Program however caused and on any theory of
--              liability. This limitation will apply even if S3 has been 
--              advised of the possibility of such damage. This limitation 
--              shall apply not-withstanding the failure of the essential 
--              purpose of any limited remedies herein.
--
--              Copyright  2005 SILICON AND SOFTWARE SYSTEMS LIMITED
--              All rights reserved-- Should get a legal disclaimer
--
--------------------------------------------------------------------------------
--
-- Silicon & Software Systems Development Record (CVS)
--
--------------------------------------------------------------------------------
--
--        $RCSfile: gnat.vhdl,v $  
--       $Revision: 1.1 $  
--           $Date: 2005/03/09 17:23:28 $  
--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--
-- Project        : Gnat
-- Project Code   : INT2004_57b
--
-- Block Name     : Gnat 
--
-- Description    : Gnat component 
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--                      Library and Package Declarations                      --
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.vcomponents.all;

library work;
use work.gnat_pkg.all;

--------------------------------------------------------------------------------
--                            Entity Description                              --
--------------------------------------------------------------------------------
entity Gnat is
   port  (
         -- BSCAN Interface
         bscan_clk            : in     std_logic;
         bscan_reset          : in     std_logic;
         bscan_sel            : in     std_logic;
         bscan_shift          : in     std_logic;
         bscan_update         : in     std_logic;
         bscan_capture        : in     std_logic;
         bscan_tdi            : in     std_logic;
         bscan_tdo            : out    std_logic;


         UserReg_Clk          : out    std_logic;
         UserReg_PeriphData   : out    std_logic_vector((c_periph_data_width - 1) downto 0);
         UserReg_PeriphOpCode : out    std_logic_vector((c_PeriphOpCode_width - 1) downto 0);  
         UserReg_PeriphSel    : out    std_logic_vector((c_num_peripherals - 1) downto 0);
         UserReg_PeriphRdData : in     t_gnat_RdData
         
         );
end entity Gnat;



architecture rtl of Gnat is
   
   signal   bscan_UserSIPO    : std_logic_vector((c_UserRegSize-1) downto 0);
   
   signal   PeripheralId      : std_logic_vector((c_PeriphID_width - 1) downto 0);
   
   signal   ReadPeriphId      : std_logic_vector((c_PeriphID_width - 1) downto 0);
   signal   ReadOpCode        : std_logic_vector((c_PeriphOpCode_width - 1) downto 0);
   signal   ReadData          : std_logic_vector((c_periph_data_width - 1) downto 0);
   signal   ReadData_mux      : std_logic_vector((c_periph_data_width - 1) downto 0);
   
begin

-- ==============================================================================
-- BSCAN User Register (SIPO)
-- ==============================================================================
   bscan_tdo  <= bscan_UserSIPO(0);
   
   ps_UserSIPO: process   (bscan_clk)
   begin
      if (bscan_clk'event and bscan_clk = '1') then
         if ( bscan_shift = '1' ) then
            for i in (c_UserRegSize - 1) downto 1 loop
               bscan_UserSIPO(i-1) <= bscan_UserSIPO(i);
            end loop;
            bscan_UserSIPO(c_UserRegSize - 1) <= bscan_tdi;
         else
            bscan_UserSIPO   <= ReadPeriphId & ReadOpCode & ReadData;
         end if;
      end if;
   end process ps_UserSIPO;   
   
-- ==============================================================================
-- Decode Peripheral from address contained in Peripheral ID field
-- ==============================================================================

   UserReg_Clk          <= bscan_update;

   PeripheralId         <= bscan_UserSIPO((c_UserRegSize - 1) downto (c_UserRegSize - c_PeriphID_width));
   UserReg_PeriphOpCode <= bscan_UserSIPO((c_UserRegSize - c_PeriphID_width - 1) downto (c_UserRegSize - c_PeriphID_width - c_PeriphOpCode_width));
   UserReg_PeriphData   <= bscan_UserSIPO((c_periph_data_width - 1) downto  0);
   
    pc_SelDecode: process   (PeripheralId, bscan_sel)
    begin
    
       if ( bscan_sel = '1') then 
          UserReg_PeriphSel   <= (others => '0');
          for i in (c_num_peripherals - 1) downto 0 loop 
             if CONV_INTEGER (PeripheralId) = i then
                UserReg_PeriphSel(i) <= '1' ;
             end if;
          end loop;  
       else
          UserReg_PeriphSel   <= (others => '0');    
       end if;      
       
    end process pc_SelDecode;      

-- ==============================================================================
-- Read Source
-- ==============================================================================
   -- The procedure for reading a Gnat Peripheral is as follows:
   -- 1. Write to USER DR with PerhiperalId of required peripheral and Read Peripheral Opcode
   -- 2. The Gnat Peripheral should return the data which is then loaded into the USER DR
   -- 3. Read the USER DR (by issuing a write to the reserved GNAT peripheral)
   -- 


   -- Capture the Periph ID and Opcode of the current peripheral being accessed.
   ps_ReadSelect: process   (bscan_update)
   begin
      if (bscan_update'event and bscan_update = '1') then
         ReadPeriphID   <= bscan_UserSIPO((c_UserRegSize - 1) downto (c_UserRegSize - c_PeriphID_width));
         ReadOpCode     <= bscan_UserSIPO((c_UserRegSize - c_PeriphID_width - 1) downto (c_UserRegSize - c_PeriphID_width - c_PeriphOpCode_width));
      end if;
   end process ps_ReadSelect;   

   -- Select the Data from the Read Peripheral
   pc_ReadMux: process   (ReadPeriphID, UserReg_PeriphRdData)
   begin
      ReadData_mux <= UserReg_PeriphRdData(0) ;
      for i in (c_num_peripherals - 1) downto 0 loop 
         if CONV_INTEGER (ReadPeriphID) = i then
            ReadData_mux <= UserReg_PeriphRdData(i) ;
         end if;
      end loop;  
   end process pc_ReadMux;   

   -- Register the Data from the Read Peripheral
   ps_ReadReg: process   (bscan_update)
   begin
      if (bscan_update'event and bscan_update = '0' ) then
         if ( bscan_sel = '1') then
            ReadData   <= ReadData_mux;
         end if;
      end if ;
   end process ps_ReadReg;   
   
end architecture rtl ;



