--
--              By downloading this software ("the Program") you agree to the
--              following  LIMITED WARRANTY AND DISCLAMER. 
--              The Program is provided to you "as is". 
--              Silicon and Software Systems Limited (S3) make, and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and S3 specifically disclaims any
--              implied warranties of merchantability, non-infringement, or
--              fitness for a particular purpose. S3 does not warrant that
--              the functions contained in the Program will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Program 
--              will be corrected. Furthermore, S3 does not warrant or
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will S3 be liable for
--              any loss of data, lost profits, cost or procurement of 
--              substitute goods or services, or for any special, incidental,
--              consequential, or indirect damages arising from the use or 
--              operation of the Program however caused and on any theory of
--              liability. This limitation will apply even if S3 has been 
--              advised of the possibility of such damage. This limitation 
--              shall apply not-withstanding the failure of the essential 
--              purpose of any limited remedies herein.
--
--              Copyright  2005 SILICON AND SOFTWARE SYSTEMS LIMITED
--              All rights reserved-- Should get a legal disclaimer
--
--------------------------------------------------------------------------------
--
-- Silicon & Software Systems Development Record (CVS)
--
--------------------------------------------------------------------------------
--
--        $RCSfile: toplevel-s3.vhdl,v $  
--       $Revision: 1.2 $  
--           $Date: 2005/03/15 11:02:41 $  
--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--
-- Project        : GNAT
-- Project Code   : INT2004_57b
--
-- Block Name     : Toplevel 
--
-- Description    : Example Toplevel architecture with 
--                   - GNAT component instantiated
--                   - BSCAN_SPARTAN3 component instantiated
--                   - BUFG component instantiated
--                   - 3 GNAT peripherals
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--                      Library and Package Declarations                      --
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.vcomponents.all;

library work;
use work.GNAT_pkg.all;

--------------------------------------------------------------------------------
--                            Entity Description                              --
--------------------------------------------------------------------------------
entity toplevel is
   port  (
	 --////////////////////////////////////////////////////////////////////////////--
	 -- XESS change:                                                               --
	 --   The width of the std_logic_vectors were changed to reflect the fact      --
	 --   that the XSA-3S1000 Board has 2 pushbuttons, 4 DIP switches, and 7 LEDs. --
	 --////////////////////////////////////////////////////////////////////////////--
         Push_Button       : in     std_logic_vector(1 downto 0);
         DIP_Switch        : in     std_logic_vector(3 downto 0);

         LED               : out    std_logic_vector(6 downto 0)
      );
end entity toplevel;

--------------------------------------------------------------------------------
--                            Architecture Description                        --
--------------------------------------------------------------------------------
architecture rtl of toplevel is

   -------------------------------------------------------------------
   --  GNAT component
   -------------------------------------------------------------------

   component GNAT
      port  (
            -- BSCAN Interface
            bscan_clk               : in     std_logic;
            bscan_reset             : in     std_logic;
            bscan_sel               : in     std_logic;
            bscan_shift             : in     std_logic;
            bscan_update            : in     std_logic;
            bscan_capture           : in     std_logic;
            bscan_tdi               : in     std_logic;
            bscan_tdo               : out    std_logic;

            UserReg_Clk             : out    std_logic;
            UserReg_PeriphData      : out    std_logic_vector((c_periph_data_width - 1) downto 0); 
            UserReg_PeriphOpcode    : out    std_logic_vector((c_PeriphOpCode_width - 1) downto 0);   
            UserReg_PeriphSel       : out    std_logic_vector((c_num_peripherals - 1) downto 0);
            UserReg_PeriphRdData    : in     t_GNAT_RdData
            );
   end component GNAT;

   -------------------------------------------------------------------
   --  BSCAN_SPARTAN3 from UNISIM Library
   -------------------------------------------------------------------

   component BSCAN_SPARTAN3
     port(
       CAPTURE : out std_ulogic ;
       DRCK1  : out std_ulogic;
       DRCK2  : out std_ulogic;
       RESET  : out std_ulogic;
       SEL1   : out std_ulogic;
       SEL2   : out std_ulogic;
       SHIFT  : out std_ulogic;
       TDI    : out std_ulogic;
       UPDATE : out std_ulogic;

       TDO1   : in  std_ulogic;
       TDO2   : in  std_ulogic
       );
   end component;

   -------------------------------------------------------------------
   --  BUFG from UNISIM Library
   -------------------------------------------------------------------

   component BUFG
     port(
       O : out std_ulogic;

       I : in  std_ulogic
       );
   end component;
  
-- Edit these constants to define the GNAT Periperal ID Mapping
-- These should match the definitions in the gnat-setup.tcl file used with the GNAT GUI
-- Note : the highest numbered Peripheral ID (in this example 15) is reserved and should not be used

   constant c_LEDRegisterID              : integer := 0 ;
   constant c_IncrementLEDRegisterID     : integer := 1 ;
   constant c_ExternalSlideSwitchesID    : integer := 2 ;
   
-- These constants define the mapping of the Peripheral Opcode Field
-- If you are using GNAT-GUI V1.0 do not change these constants

   constant c_ReadOpCode                 : integer := 1 ;
   constant c_WriteOpCode                : integer := 0 ;
   
   -- Signal Declarations

   signal   bscan_capture                 : std_logic;
   signal   bscan_drck1                   : std_logic;
   signal   bscan_drck2                   : std_logic;
   signal   bscan_reset                   : std_logic;
   signal   bscan_sel1                    : std_logic;
   signal   bscan_sel2                    : std_logic;
   signal   bscan_shift                   : std_logic;
   signal   bscan_tdi                     : std_logic;
   signal   bscan_update                  : std_logic;
   signal   bscan_tdo1                    : std_logic;
   signal   bscan_tdo2                    : std_logic;

   signal   buffered_bscan_drck2          : std_logic;

   signal   UserReg_Clk                   : std_logic;
   signal   UserReg_PeriphData            : std_logic_vector((c_periph_data_width - 1) downto 0); 
   signal   UserReg_PeriphOpcode          : std_logic_vector((c_PeriphOpCode_width - 1) downto 0);  
   signal   UserReg_PeriphSel             : std_logic_vector((c_num_peripherals - 1) downto 0);
   signal   UserReg_PeriphRdData          : t_GNAT_RdData;

   --/////////////////////////////////////////////////////////////////
   -- XESS change:
   --   Set the size of the LED output register so it always matches
   --   the size of the LED output std_logic_vector.
   --/////////////////////////////////////////////////////////////////
   signal   led_output_int                : std_logic_vector(LED'range) ;
   signal   ResetSwitchRegister           : std_logic; 
  
   signal   PeriphWriteStrobe             : std_logic;
   signal   PeriphReadStrobe              : std_logic;
            
       
begin

--------------------------------------------------------------------------------
-- Instantiate BSCAN, BUFG & GNAT Components
-- Note in this example the GNAT component is wired to USER2 of the BSCAN component
--------------------------------------------------------------------------------

   i_bscan : BSCAN_SPARTAN3
      port map (
               CAPTURE => bscan_capture,
               DRCK1   => bscan_drck1,
               DRCK2   => bscan_drck2,
               RESET   => bscan_reset,
               SEL1    => bscan_sel1, 
               SEL2    => bscan_sel2,
               SHIFT   => bscan_shift,
               TDI     => bscan_tdi,
               UPDATE  => bscan_update,

               TDO1    => bscan_tdo1,
               TDO2    => bscan_tdo2
               );

   i00_BUFG : BUFG
      port map (
               O => buffered_bscan_drck2,
               I => bscan_drck2
               );

   i00_GNAT: GNAT
      port map (
               bscan_clk                  => buffered_bscan_drck2,
               bscan_reset                => bscan_reset,
               bscan_sel                  => bscan_sel2,
               bscan_shift                => bscan_shift,
               bscan_update               => bscan_update,
               bscan_capture              => bscan_capture,
               bscan_tdi                  => bscan_tdi,
               bscan_tdo                  => bscan_tdo2,
               
               UserReg_Clk                => UserReg_Clk,
               UserReg_PeriphData         => UserReg_PeriphData,
               UserReg_PeriphOpcode       => UserReg_PeriphOpcode,
               UserReg_PeriphSel          => UserReg_PeriphSel,
               UserReg_PeriphRdData       => UserReg_PeriphRdData
               );
  
--------------------------------------------------------------------------------
-- Three GNAT Periperals defined
--    
--    ID 0. LEDRegister , 8 bit read/write by GNAT Component
--          - value loaded by GNAT 
--          - incremented by GNAT peripheral IncrementLEDRegister
--          - reset by external pushbutton (0) 
--    ID 1. IncrementLEDRegister  , 1 bit write only by GNAT Component, active high
--    ID 2. ExternalSlideSwitches , 8 bit read only by GNAT Component
--
--------------------------------------------------------------------------------

-- Decode required peripheral operation from UserReg_PeriphOpcode
   pc_PeriphOpDecode : process (UserReg_PeriphOpcode)
   begin
      PeriphWriteStrobe <= '0' ;
      PeriphReadStrobe  <= '0' ;
      if (CONV_INTEGER (UserReg_PeriphOpcode) = c_WriteOpCode) then
         PeriphWriteStrobe <= '1' ;   
      end if ;
      if (CONV_INTEGER (UserReg_PeriphOpcode) = c_ReadOpCode) then
         PeriphReadStrobe <= '1' ;   
      end if ;      
   end process pc_PeriphOpDecode ;   

-- LEDRegister with IncrementLEDRegister
   ps_LedRegister : process (UserReg_Clk, Push_Button)
   begin
      --/////////////////////////////////////////////////////////////
      -- XESS change:
      --   The XSA Board pushbuttons are active-low.
      --/////////////////////////////////////////////////////////////
      if Push_Button(0) = '0' then
         led_output_int <= (others => '0');
      elsif rising_edge(UserReg_Clk) then   
         if (UserReg_PeriphSel(c_LEDRegisterID) = '1' and PeriphWriteStrobe = '1') then 
            --/////////////////////////////////////////////////////////////////////////
            -- XESS change:
            --   The range of the LED output register was appended to the
            --   peripheral data register because there are only 7 LEDs and the data
            --   register is 8 bits wide.  So just use the lower 7 bits.
            --/////////////////////////////////////////////////////////////////////////
            led_output_int <= UserReg_PeriphData(led_output_int'range) ;
         elsif (UserReg_PeriphSel(c_IncrementLEDRegisterID) = '1' and PeriphWriteStrobe = '1' and UserReg_PeriphData(0) ='1') then 
            led_output_int <= led_output_int + 1;
         end if ;
      end if ;
   end process ps_LedRegister; 
   
   --//////////////////////////////////////////////////////////////////////////
   -- XESS change:
   --   The range of the LED output register was appended to the
   --   peripheral data register because there are only 7 LEDs and the data
   --   register is 8 bits wide.  So just use the lower 7 bits.
   --//////////////////////////////////////////////////////////////////////////
   UserReg_PeriphRdData(c_LEDRegisterID)(led_output_int'range) <= led_output_int;
   
   LED   <= led_output_int; 
       
-- ExternalSlideSwitches
   --//////////////////////////////////////////////////////////////////////////
   -- XESS change:
   -- The range of the DIP switch register was appended to the
   -- peripheral data register because there are only 4 switches and the data
   -- register is 8 bits wide.  So just use the lower 4 bits.
   --//////////////////////////////////////////////////////////////////////////
   UserReg_PeriphRdData(c_ExternalSlideSwitchesID)(DIP_switch'range) <= DIP_Switch;

end architecture rtl ;


